# -*- coding: utf-8 -*-

from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile

from Products.PDFNode.PDFDocument import PDFDocument

VIEW_PERMISSION = 'View'


manage_addPDFNodeForm=DTMLFile('dtml/addPDFNode_form', globals())


def manage_addPDFNode( self, id, REQUEST=None ):
    """
    Add a PDFNode into the system
    """

    obj = PDFNode(id)
    self._setObject(id, obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

add = manage_addPDFNode



class PDFNode(PropertyManager, SimpleItem):
    """
    This tool encapsulates PDF generation.
    """

    meta_type = "PDFNode"
    id = "PDFNode"
    title = "PDFNode pdf generator"
    icon_file = 'pdfnode_icon.png'    
    
    security = ClassSecurityInfo()

    manage_options = PropertyManager.manage_options + SimpleItem.manage_options

    _properties = (
        {'id': 'attachment_section', 'type': 'string', 'mode': 'rw', "label":"Default attachment section"},
        {'id': 'pdflatex', 'type': 'string', 'mode': 'rw', "label":"Path of pdflatex"},
        {'id': 'htmldoc', 'type': 'string', 'mode': 'rw', "label":"Path of htmldoc"},
        {'id': 'textoc', 'type': 'boolean', 'mode': 'rw', "label":"Include LaTeX TOC"},
        {'id': 'htmltoc', 'type': 'boolean', 'mode': 'rw', "label":"Include htmldoc TOC"},
        {'id': 'htmlnumbertoc', 'type': 'boolean', 'mode': 'rw',  "label":"Number htmldoc TOC"},
        {'id': 'papersize', 'type': 'string', 'mode': 'rw', "label":"LaTeX paper size code"},
        {'id': 'htmlpapersize', 'type': 'string', 'mode': 'rw', "label":"htmldoc paper size code"},
        {'id': 'orientation', 'type': 'string', 'mode': 'rw', "label":"Paper orientation"},
        {'id': 'cleanup', 'type': 'boolean', 'mode': 'rw', "label":"Clean temporary files"},
        {'id': 'documentclass', 'type': 'string', 'mode': 'rw', "label":"LaTeX document class"},
        {'id': 'level1', 'type': 'string', 'mode': 'rw', "label":"LaTeX level 1 headings"},
        {'id': 'level2', 'type': 'string', 'mode': 'rw', "label":"LaTeX level 2 headings"},
        {'id': 'htmldoc_options', 'type': 'string', 'mode': 'rw', "label":"More htmldoc options"},
        {'id': 'extra_headers', 'type': 'text', 'mode': 'rw', "label":"More LaTeX headers"},
        )

    attachment_section = "Appendixes"
    pdflatex = "pdflatex"
    htmldoc = "htmldoc"
    textoc = False
    htmltoc = False
    htmlnumbertoc = True
    papersize = "a4paper"
    htmlpapersize = "a4"
    orientation = "portrait"
    cleanup = True
    documentclass = "book"
    level1 = "part*"
    level2 = "chapter*"    
    htmldoc_options = "--no-title --color --format pdf13"
    extra_headers = ""

    security.declareProtected(VIEW_PERMISSION, "renderAsPdf")
    def renderAsPdf(self, content, name, encoding = "utf-8",
                    attachments = [], source_object = None):
        """
        All-in-one method to render a simple HTML document,
        with some PDF attachements, in PDF
        """
        document = self.newPDF()
        document.add_html(content, encoding, 
                            source_object = source_object)
        for attachement in attachments:
            document.add_attachment(attachement, 
                                        self.attachment_section)
        self.generate(document, name)

    security.declareProtected(VIEW_PERMISSION, 'newPDF')
    def newPDF(self):
        """
        Create a new PDF object
        """
        document = PDFDocument(context = self)
        document.configure(pdflatex = self.pdflatex,
                           htmldoc = self.htmldoc,
                           textoc = self.textoc,
                           htmltoc = self.htmltoc,
                           htmlnumbertoc = self.htmlnumbertoc,
                           papersize = self.papersize,
                           htmlpapersize = self.htmlpapersize,
                           orientation = self.orientation,
                           cleanup = self.cleanup,
                           documentclass = self.documentclass,
                           level1 = self.level1,
                           level2 = self.level2,
                           htmldoc_options = self.htmldoc_options,
                           extra_headers = self.extra_headers)
        return document

    security.declareProtected(VIEW_PERMISSION, 'generate')
    def generate(self, document, name):
        """
        Generate a document
        """
        pdf_content = document.generate()
        out_name = name + '.pdf'
        self.REQUEST.RESPONSE.setHeader('Content-type', 'application/pdf')
        self.REQUEST.RESPONSE.setHeader('Content-length', str(len(pdf_content)))
        self.REQUEST.RESPONSE.setHeader('Content-disposition', 'attachment; filename=%s' % out_name)
        return pdf_content

InitializeClass(PDFNode)
