# -*- coding: utf-8 -*-

from PDFRenderer import PDFRenderer
import os
import re

class HTMLRenderer(PDFRenderer):
    """
    HTML renderer class
    """
    mimetype = "text/html"

    def generate_pdf(self):
        """
        Generate PDF file
        """
        if self.config['source_object']:
            # prepare for image rendering
            self.fetchImages(self.config['source_object'],self.partname+'_')

        in_file = os.path.join(self.config["tmpdir"], self.partname + ".html")
        f = open(in_file, 'w')
        f.write(self.data)
        f.close()
        
        out_file = os.path.join(self.config["tmpdir"], self.partname + ".pdf")
        command = "%(htmldoc)s %(htmldoc_options)s --no-title --color --format pdf13 --size %(htmlpapersize)s " % self.config
        if self.config["htmlnumbertoc"]:
            command += "--numbered "
        if not self.config["htmltoc"]:
            command += "--no-toc "

        os.system("%s --outfile %s %s" % (command, out_file, in_file))
        # Try --webpage if nothing is generated
        if not os.path.exists(out_file) and not "--webpage" in command:
            os.system("%s --webpage --outfile %s %s" % (command, out_file, in_file))

        return out_file
        
    def fetchImages(self,source_object,filename_prefix):
        regex_image = '<img%ssrc="%s"%s/>'
        
        # Retrieve images files
        portal = source_object.portal_url.getPortalObject()
        image_ref_regexp = re.compile('<img(.*?)src="(.*?)"(.*?)/>', \
                           re.IGNORECASE + re.DOTALL)
        image_balises = re.findall(image_ref_regexp,self.data)
        
        image_cpt = 0 # in order to 
        for image_balise in image_balises:
            image = None
            if image_balise[1].find(source_object.portal_url()) == 0:
                # absolute URL
                # image in this site, retrieve the relative path
                relative_path = image_balise[1][len(source_object.portal_url()):]
                image = self._getObjectFromPath(portal,relative_path)

            elif image_balise[1].find('http://') != 0:
                # relative URL
                # image in this site, retrieve the relative path
                relative_path = image_balise[1]
                image = self._getObjectFromPath(source_object,image_balise[1])

            if image:
                # write image to disk
                image_cpt += 1
                extension = ''
                if '.' in relative_path :
                    extension = '.'+relative_path.split('.')[-1]
                file_name = filename_prefix+str(image_cpt)+extension
                image_file = open(self.config["tmpdir"] + os.sep + file_name,'w')
                if hasattr(image,'_data'):
                    image_file.write(str(image._data))
                else:
                    image_file.write(str(image.data))
                image_file.flush()
                image_file.close()

                # replace image path in balise
                self.data = self.data.replace(regex_image % tuple(image_balise), \
                                regex_image % (image_balise[0], \
                                file_name, \
                                image_balise[2]) )
    

    def _getObjectFromPath(self,portal,rPath):
        # return None if object is not found
        path_tab = rPath.strip('/').split('/')
        cur_object = portal
        for path_element in path_tab:
            cur_object = getattr(cur_object,path_element,None)
            if not cur_object:
                return None
        return cur_object

